#ifndef QT_QTWEBKITWIDGETS_VERSION_H
#define QT_QTWEBKITWIDGETS_VERSION_H

#define QTWEBKITWIDGETS_VERSION_STR "5.212.0"
#define QTWEBKITWIDGETS_VERSION_MAJOR 5
#define QTWEBKITWIDGETS_VERSION_MINOR 212
#define QTWEBKITWIDGETS_VERSION_PATCH 0
#define QTWEBKITWIDGETS_VERSION ((5<<16)|(212<<8)|(0))

#endif
